# Copyright 2019 NetApp, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manila import context
from manila.db import api as db_api
from manila.scheduler.weighers import base_host
from manila.share import utils as share_utils


class HostAffinityWeigher(base_host.BaseHostWeigher):

    def _weigh_object(self, obj, weight_properties):
        """Weigh hosts based on their proximity to the source's share pool.

        If no snapshot_id was provided will return 0, otherwise, if source and
        destination hosts are located on:
        1. same back ends and pools: host is a perfect choice (100)
        2. same back ends and different pools: host is a very good choice (75)
        3. different back ends with the same AZ: host is a good choice (50)
        4. different back ends and AZs: host isn't so good choice (25)
        """

        ctx = context.get_admin_context()
        request_spec = weight_properties.get('request_spec')
        snapshot_id = request_spec.get('snapshot_id')
        snapshot_host = request_spec.get('snapshot_host')

        if None in [snapshot_id, snapshot_host]:
            # NOTE(silvacarlose): if the request does not contain a snapshot_id
            # or a snapshot_host, the user is not creating a share from a
            # snapshot and we don't need to weigh the host.
            return 0

        snapshot_ref = db_api.share_snapshot_get(ctx, snapshot_id)
        # Source host info: pool, backend and availability zone
        src_pool = share_utils.extract_host(snapshot_host, 'pool')
        src_backend = share_utils.extract_host(
            request_spec.get('snapshot_host'), 'backend')
        src_az = snapshot_ref['share']['availability_zone']
        # Destination host info: pool, backend and availability zone
        dst_pool = share_utils.extract_host(obj.host, 'pool')
        dst_backend = share_utils.extract_host(obj.host, 'backend')
        # NOTE(dviroel): All hosts were already filtered by the availability
        # zone parameter.
        dst_az = None
        if weight_properties['availability_zone_id']:
            dst_az = db_api.availability_zone_get(
                ctx, weight_properties['availability_zone_id']).name

        if src_backend == dst_backend:
            return 100 if (src_pool and src_pool == dst_pool) else 75
        else:
            return 50 if (src_az and src_az == dst_az) else 25



# Copyright (c) 2016 by Tegile Systems, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Share driver Test for Tegile storage.
"""

from unittest import mock

import ddt
from oslo_config import cfg
import requests
import six

from manila.common import constants as const
from manila import context
from manila import exception
from manila.share import configuration
from manila.share.drivers.tegile import tegile
from manila import test


CONF = cfg.CONF

test_config = configuration.Configuration(None)
test_config.tegile_nas_server = 'some-ip'
test_config.tegile_nas_login = 'some-user'
test_config.tegile_nas_password = 'some-password'
test_config.reserved_share_percentage = 10
test_config.reserved_share_from_snapshot_percentage = 5
test_config.max_over_subscription_ratio = 30.0

test_share = {
    'host': 'node#fake_pool',
    'name': 'testshare',
    'id': 'a24c2ee8-525a-4406-8ccd-8d38688f8e9e',
    'share_proto': 'NFS',
    'size': 10,
}

test_share_cifs = {
    'host': 'node#fake_pool',
    'name': 'testshare',
    'id': 'a24c2ee8-525a-4406-8ccd-8d38688f8e9e',
    'share_proto': 'CIFS',
    'size': 10,
}

test_share_fail = {
    'host': 'node#fake_pool',
    'name': 'testshare',
    'id': 'a24c2ee8-525a-4406-8ccd-8d38688f8e9e',
    'share_proto': 'OTHER',
    'size': 10,
}

test_snapshot = {
    'name': 'testSnap',
    'id': '07ae9978-5445-405e-8881-28f2adfee732',
    'share': test_share,
    'share_name': 'snapshotted',
    'display_name': 'disp',
    'display_description': 'disp-desc',
}

array_stats = {
    'total_capacity_gb': 4569.199686084874,
    'free_capacity_gb': 4565.381390112452,
    'pools': [
        {
            'total_capacity_gb': 913.5,
            'QoS_support': False,
            'free_capacity_gb': 911.812650680542,
            'reserved_percentage': 0,
            'reserved_snapshot_percentage': 0,
            'pool_name': 'pyramid',
        },
        {
            'total_capacity_gb': 2742.1996604874,
            'QoS_support': False,
            'free_capacity_gb': 2740.148867149747,
            'reserved_percentage': 0,
            'reserved_snapshot_percentage': 0,
            'pool_name': 'cobalt',
        },
        {
            'total_capacity_gb': 913.5,
            'QoS_support': False,
            'free_capacity_gb': 913.4198722839355,
            'reserved_percentage': 0,
            'reserved_snapshot_percentage': 0,
            'pool_name': 'test',
        },
    ],
}


fake_tegile_backend_fail = mock.Mock(
    side_effect=exception.TegileAPIException(response="Fake Exception"))


class FakeResponse(object):
    def __init__(self, status, json_output):
        self.status_code = status
        self.text = 'Random text'
        self._json = json_output

    def json(self):
        return self._json

    def close(self):
        pass


@ddt.ddt
class TegileShareDriverTestCase(test.TestCase):
    def __init__(self, *args, **kwds):
        super(TegileShareDriverTestCase, self).__init__(*args, **kwds)
        self._ctxt = context.get_admin_context()
        self.configuration = test_config

    def setUp(self):
        CONF.set_default('driver_handles_share_servers', False)
        self._driver = tegile.TegileShareDriver(
            configuration=self.configuration)
        self._driver._default_project = 'fake_project'
        super(TegileShareDriverTestCase, self).setUp()

    def test_create_share(self):
        api_return_value = (test_config.tegile_nas_server +
                            " " + test_share['name'])
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        return_value=api_return_value))

        result = self._driver.create_share(self._ctxt, test_share)

        expected = {
            'is_admin_only': False,
            'metadata': {
                'preferred': True,
            },
            'path': 'some-ip:testshare',
        }
        self.assertEqual(expected, result)

        create_params = (
            'fake_pool',
            'fake_project',
            test_share['name'],
            test_share['share_proto'],
        )
        mock_api.assert_called_once_with('createShare', create_params)

    def test_create_share_fail(self):
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        side_effect=(
                                            exception.TegileAPIException(
                                                response="Fake Exception"))))

        self.assertRaises(exception.TegileAPIException,
                          self._driver.create_share,
                          self._ctxt,
                          test_share)

        create_params = (
            'fake_pool',
            'fake_project',
            test_share['name'],
            test_share['share_proto'],
        )
        mock_api.assert_called_once_with('createShare', create_params)

    def test_delete_share(self):
        fake_share_info = ('fake_pool', 'fake_project', test_share['name'])
        mock_params = self.mock_object(self._driver,
                                       '_get_pool_project_share_name',
                                       mock.Mock(return_value=fake_share_info))
        mock_api = self.mock_object(self._driver, '_api')

        self._driver.delete_share(self._ctxt, test_share)

        delete_path = '%s/%s/%s/%s' % (
            'fake_pool', 'Local', 'fake_project', test_share['name'])
        delete_params = (delete_path, True, False)
        mock_api.assert_called_once_with('deleteShare', delete_params)
        mock_params.assert_called_once_with(test_share)

    def test_delete_share_fail(self):
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        side_effect=(
                                            exception.TegileAPIException(
                                                response="Fake Exception"))))

        self.assertRaises(exception.TegileAPIException,
                          self._driver.delete_share,
                          self._ctxt,
                          test_share)

        delete_path = '%s/%s/%s/%s' % (
            'fake_pool', 'Local', 'fake_project', test_share['name'])
        delete_params = (delete_path, True, False)
        mock_api.assert_called_once_with('deleteShare', delete_params)

    def test_create_snapshot(self):
        mock_api = self.mock_object(self._driver, '_api')
        fake_share_info = ('fake_pool', 'fake_project', test_share['name'])
        mock_params = self.mock_object(self._driver,
                                       '_get_pool_project_share_name',
                                       mock.Mock(return_value=fake_share_info))

        self._driver.create_snapshot(self._ctxt, test_snapshot)

        share = {
            'poolName': 'fake_pool',
            'projectName': 'fake_project',
            'name': test_share['name'],
            'availableSize': 0,
            'totalSize': 0,
            'datasetPath': '%s/%s/%s' % (
                'fake_pool',
                'Local',
                'fake_project',
            ),
            'mountpoint': test_share['name'],
            'local': 'true',
        }
        create_params = (share, test_snapshot['name'], False)
        mock_api.assert_called_once_with('createShareSnapshot', create_params)
        mock_params.assert_called_once_with(test_share)

    def test_create_snapshot_fail(self):
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        side_effect=(
                                            exception.TegileAPIException(
                                                response="Fake Exception"))))

        self.assertRaises(exception.TegileAPIException,
                          self._driver.create_snapshot,
                          self._ctxt,
                          test_snapshot)

        share = {
            'poolName': 'fake_pool',
            'projectName': 'fake_project',
            'name': test_share['name'],
            'availableSize': 0,
            'totalSize': 0,
            'datasetPath': '%s/%s/%s' % (
                'fake_pool',
                'Local',
                'fake_project',
            ),
            'mountpoint': test_share['name'],
            'local': 'true',
        }
        create_params = (share, test_snapshot['name'], False)
        mock_api.assert_called_once_with('createShareSnapshot', create_params)

    def test_delete_snapshot(self):
        fake_share_info = ('fake_pool', 'fake_project', test_share['name'])
        mock_params = self.mock_object(self._driver,
                                       '_get_pool_project_share_name',
                                       mock.Mock(return_value=fake_share_info))
        mock_api = self.mock_object(self._driver, '_api')

        self._driver.delete_snapshot(self._ctxt, test_snapshot)

        delete_snap_path = ('%s/%s/%s/%s@%s%s' % (
            'fake_pool',
            'Local',
            'fake_project',
            test_share['name'],
            'Manual-S-',
            test_snapshot['name'],
        ))

        delete_params = (delete_snap_path, False)
        mock_api.assert_called_once_with('deleteShareSnapshot', delete_params)
        mock_params.assert_called_once_with(test_share)

    def test_delete_snapshot_fail(self):
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        side_effect=(
                                            exception.TegileAPIException(
                                                response="Fake Exception"))))

        self.assertRaises(exception.TegileAPIException,
                          self._driver.delete_snapshot,
                          self._ctxt,
                          test_snapshot)

        delete_snap_path = ('%s/%s/%s/%s@%s%s' % (
            'fake_pool',
            'Local',
            'fake_project',
            test_share['name'],
            'Manual-S-',
            test_snapshot['name'],
        ))
        delete_params = (delete_snap_path, False)
        mock_api.assert_called_once_with('deleteShareSnapshot', delete_params)

    def test_create_share_from_snapshot(self):
        api_return_value = (test_config.tegile_nas_server +
                            " " + test_share['name'])
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        return_value=api_return_value))
        fake_share_info = ('fake_pool', 'fake_project', test_share['name'])
        mock_params = self.mock_object(self._driver,
                                       '_get_pool_project_share_name',
                                       mock.Mock(return_value=fake_share_info))

        result = self._driver.create_share_from_snapshot(self._ctxt,
                                                         test_share,
                                                         test_snapshot)

        expected = {
            'is_admin_only': False,
            'metadata': {
                'preferred': True,
            },
            'path': 'some-ip:testshare',
        }
        self.assertEqual(expected, result)

        create_params = (
            '%s/%s/%s/%s@%s%s' % (
                'fake_pool',
                'Local',
                'fake_project',
                test_snapshot['share_name'],
                'Manual-S-',
                test_snapshot['name'],
            ),
            test_share['name'],
            True,
        )
        mock_api.assert_called_once_with('cloneShareSnapshot', create_params)
        mock_params.assert_called_once_with(test_share)

    def test_create_share_from_snapshot_fail(self):
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        side_effect=(
                                            exception.TegileAPIException(
                                                response="Fake Exception"))))

        self.assertRaises(exception.TegileAPIException,
                          self._driver.create_share_from_snapshot,
                          self._ctxt,
                          test_share,
                          test_snapshot)

        create_params = (
            '%s/%s/%s/%s@%s%s' % (
                'fake_pool',
                'Local',
                'fake_project',
                test_snapshot['share_name'],
                'Manual-S-',
                test_snapshot['name'],
            ),
            test_share['name'],
            True,
        )
        mock_api.assert_called_once_with('cloneShareSnapshot', create_params)

    def test_ensure_share(self):
        api_return_value = (test_config.tegile_nas_server +
                            " " + test_share['name'])
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        return_value=api_return_value))
        fake_share_info = ('fake_pool', 'fake_project', test_share['name'])
        mock_params = self.mock_object(self._driver,
                                       '_get_pool_project_share_name',
                                       mock.Mock(return_value=fake_share_info))

        result = self._driver.ensure_share(self._ctxt, test_share)

        expected = [
            {
                'is_admin_only': False,
                'metadata': {
                    'preferred':
                        True,
                },
                'path': 'some-ip:testshare',
            },
        ]
        self.assertEqual(expected, result)

        ensure_params = [
            '%s/%s/%s/%s' % (
                'fake_pool', 'Local', 'fake_project', test_share['name'])]
        mock_api.assert_called_once_with('getShareIPAndMountPoint',
                                         ensure_params)
        mock_params.assert_called_once_with(test_share)

    def test_ensure_share_fail(self):
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        side_effect=(
                                            exception.TegileAPIException(
                                                response="Fake Exception"))))
        self.assertRaises(exception.TegileAPIException,
                          self._driver.ensure_share,
                          self._ctxt,
                          test_share)

        ensure_params = [
            '%s/%s/%s/%s' % (
                'fake_pool', 'Local', 'fake_project', test_share['name'])]
        mock_api.assert_called_once_with('getShareIPAndMountPoint',
                                         ensure_params)

    def test_get_share_stats(self):
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        return_value=array_stats))

        result_dict = self._driver.get_share_stats(True)

        expected_dict = {
            'driver_handles_share_servers': False,
            'driver_version': '1.0.0',
            'free_capacity_gb': 4565.381390112452,
            'pools': [
                {
                    'allocated_capacity_gb': 0.0,
                    'compression': True,
                    'dedupe': True,
                    'free_capacity_gb': 911.812650680542,
                    'pool_name': 'pyramid',
                    'qos': False,
                    'reserved_percentage': 10,
                    'reserved_snapshot_percentage': 5,
                    'thin_provisioning': True,
                    'max_over_subscription_ratio': 30.0,
                    'total_capacity_gb': 913.5},
                {
                    'allocated_capacity_gb': 0.0,
                    'compression': True,
                    'dedupe': True,
                    'free_capacity_gb': 2740.148867149747,
                    'pool_name': 'cobalt',
                    'qos': False,
                    'reserved_percentage': 10,
                    'reserved_snapshot_percentage': 5,
                    'thin_provisioning': True,
                    'max_over_subscription_ratio': 30.0,
                    'total_capacity_gb': 2742.1996604874
                },
                {
                    'allocated_capacity_gb': 0.0,
                    'compression': True,
                    'dedupe': True,
                    'free_capacity_gb': 913.4198722839355,
                    'pool_name': 'test',
                    'qos': False,
                    'reserved_percentage': 10,
                    'reserved_snapshot_percentage': 5,
                    'thin_provisioning': True,
                    'max_over_subscription_ratio': 30.0,
                    'total_capacity_gb': 913.5}, ],
            'qos': False,
            'reserved_percentage': 0,
            'reserved_snapshot_percentage': 0,
            'replication_domain': None,
            'share_backend_name': 'Tegile',
            'snapshot_support': True,
            'storage_protocol': 'NFS_CIFS',
            'total_capacity_gb': 4569.199686084874,
            'vendor_name': 'Tegile Systems Inc.',
        }
        self.assertSubDictMatch(expected_dict, result_dict)

        mock_api.assert_called_once_with(fine_logging=False,
                                         method='getArrayStats',
                                         request_type='get')

    def test_get_share_stats_fail(self):
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        side_effect=(
                                            exception.TegileAPIException(
                                                response="Fake Exception"))))

        self.assertRaises(exception.TegileAPIException,
                          self._driver.get_share_stats,
                          True)

        mock_api.assert_called_once_with(fine_logging=False,
                                         method='getArrayStats',
                                         request_type='get')

    def test_get_pool(self):
        result = self._driver.get_pool(test_share)

        expected = 'fake_pool'
        self.assertEqual(expected, result)

    def test_extend_share(self):
        fake_share_info = ('fake_pool', 'fake_project', test_share['name'])
        mock_params = self.mock_object(self._driver,
                                       '_get_pool_project_share_name',
                                       mock.Mock(return_value=fake_share_info))
        mock_api = self.mock_object(self._driver, '_api')

        self._driver.extend_share(test_share, 12)

        extend_path = '%s/%s/%s/%s' % (
            'fake_pool', 'Local', 'fake_project', test_share['name'])
        extend_params = (extend_path, six.text_type(12), 'GB')
        mock_api.assert_called_once_with('resizeShare', extend_params)
        mock_params.assert_called_once_with(test_share)

    def test_extend_share_fail(self):
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        side_effect=(
                                            exception.TegileAPIException(
                                                response="Fake Exception"))))

        self.assertRaises(exception.TegileAPIException,
                          self._driver.extend_share,
                          test_share, 30)

        extend_path = '%s/%s/%s/%s' % (
            'fake_pool', 'Local', 'fake_project', test_share['name'])
        extend_params = (extend_path, six.text_type(30), 'GB')
        mock_api.assert_called_once_with('resizeShare', extend_params)

    def test_shrink_share(self):
        fake_share_info = ('fake_pool', 'fake_project', test_share['name'])
        mock_params = self.mock_object(self._driver,
                                       '_get_pool_project_share_name',
                                       mock.Mock(return_value=fake_share_info))
        mock_api = self.mock_object(self._driver, '_api')

        self._driver.shrink_share(test_share, 15)

        shrink_path = '%s/%s/%s/%s' % (
            'fake_pool', 'Local', 'fake_project', test_share['name'])
        shrink_params = (shrink_path, six.text_type(15), 'GB')
        mock_api.assert_called_once_with('resizeShare', shrink_params)
        mock_params.assert_called_once_with(test_share)

    def test_shrink_share_fail(self):
        mock_api = self.mock_object(self._driver, '_api',
                                    mock.Mock(
                                        side_effect=(
                                            exception.TegileAPIException(
                                                response="Fake Exception"))))

        self.assertRaises(exception.TegileAPIException,
                          self._driver.shrink_share,
                          test_share, 30)

        shrink_path = '%s/%s/%s/%s' % (
            'fake_pool', 'Local', 'fake_project', test_share['name'])
        shrink_params = (shrink_path, six.text_type(30), 'GB')
        mock_api.assert_called_once_with('resizeShare', shrink_params)

    @ddt.data('ip', 'user')
    def test_allow_access(self, access_type):
        fake_share_info = ('fake_pool', 'fake_project', test_share['name'])
        mock_params = self.mock_object(self._driver,
                                       '_get_pool_project_share_name',
                                       mock.Mock(return_value=fake_share_info))
        mock_api = self.mock_object(self._driver, '_api')

        access = {
            'access_type': access_type,
            'access_level': const.ACCESS_LEVEL_RW,
            'access_to': 'some-ip',
        }

        self._driver._allow_access(self._ctxt, test_share, access)

        allow_params = (
            '%s/%s/%s/%s' % (
                'fake_pool',
                'Local',
                'fake_project',
                test_share['name'],
            ),
            test_share['share_proto'],
            access_type,
            access['access_to'],
            access['access_level'],
        )
        mock_api.assert_called_once_with('shareAllowAccess', allow_params)
        mock_params.assert_called_once_with(test_share)

    @ddt.data({'access_type': 'other', 'to': 'some-ip', 'share': test_share,
               'exception_type': exception.InvalidShareAccess},
              {'access_type': 'ip', 'to': 'some-ip', 'share': test_share,
               'exception_type': exception.TegileAPIException},
              {'access_type': 'ip', 'to': 'some-ip', 'share': test_share_cifs,
               'exception_type': exception.InvalidShareAccess},
              {'access_type': 'ip', 'to': 'some-ip', 'share': test_share_fail,
               'exception_type': exception.InvalidShareAccess})
    @ddt.unpack
    def test_allow_access_fail(self, access_type, to, share, exception_type):
        self.mock_object(self._driver, '_api',
                         mock.Mock(
                             side_effect=exception.TegileAPIException(
                                 response="Fake Exception")))

        access = {
            'access_type': access_type,
            'access_level': const.ACCESS_LEVEL_RW,
            'access_to': to,
        }

        self.assertRaises(exception_type,
                          self._driver._allow_access,
                          self._ctxt,
                          share,
                          access)

    @ddt.data('ip', 'user')
    def test_deny_access(self, access_type):
        fake_share_info = ('fake_pool', 'fake_project', test_share['name'])
        mock_params = self.mock_object(self._driver,
                                       '_get_pool_project_share_name',
                                       mock.Mock(return_value=fake_share_info))
        mock_api = self.mock_object(self._driver, '_api')

        access = {
            'access_type': access_type,
            'access_level': const.ACCESS_LEVEL_RW,
            'access_to': 'some-ip',
        }

        self._driver._deny_access(self._ctxt, test_share, access)

        deny_params = (
            '%s/%s/%s/%s' % (
                'fake_pool',
                'Local',
                'fake_project',
                test_share['name'],
            ),
            test_share['share_proto'],
            access_type,
            access['access_to'],
            access['access_level'],
        )
        mock_api.assert_called_once_with('shareDenyAccess', deny_params)
        mock_params.assert_called_once_with(test_share)

    @ddt.data({'access_type': 'other', 'to': 'some-ip', 'share': test_share,
               'exception_type': exception.InvalidShareAccess},
              {'access_type': 'ip', 'to': 'some-ip', 'share': test_share,
               'exception_type': exception.TegileAPIException},
              {'access_type': 'ip', 'to': 'some-ip', 'share': test_share_cifs,
               'exception_type': exception.InvalidShareAccess},
              {'access_type': 'ip', 'to': 'some-ip', 'share': test_share_fail,
               'exception_type': exception.InvalidShareAccess})
    @ddt.unpack
    def test_deny_access_fail(self, access_type, to, share, exception_type):
        self.mock_object(self._driver, '_api',
                         mock.Mock(
                             side_effect=exception.TegileAPIException(
                                 response="Fake Exception")))

        access = {
            'access_type': access_type,
            'access_level': const.ACCESS_LEVEL_RW,
            'access_to': to,
        }

        self.assertRaises(exception_type,
                          self._driver._deny_access,
                          self._ctxt,
                          share,
                          access)

    @ddt.data({'access_rules': [{'access_type': 'ip',
                                 'access_level': const.ACCESS_LEVEL_RW,
                                 'access_to': 'some-ip',
                                 }, ], 'add_rules': None,
               'delete_rules': None, 'call_name': 'shareAllowAccess'},
              {'access_rules': [], 'add_rules':
                  [{'access_type': 'ip',
                    'access_level': const.ACCESS_LEVEL_RW,
                    'access_to': 'some-ip'}, ], 'delete_rules': [],
               'call_name': 'shareAllowAccess'},
              {'access_rules': [], 'add_rules': [], 'delete_rules':
                  [{'access_type': 'ip',
                    'access_level': const.ACCESS_LEVEL_RW,
                    'access_to': 'some-ip', }, ],
               'call_name': 'shareDenyAccess'})
    @ddt.unpack
    def test_update_access(self, access_rules, add_rules,
                           delete_rules, call_name):
        fake_share_info = ('fake_pool', 'fake_project', test_share['name'])
        mock_params = self.mock_object(self._driver,
                                       '_get_pool_project_share_name',
                                       mock.Mock(return_value=fake_share_info))
        mock_api = self.mock_object(self._driver, '_api')

        self._driver.update_access(self._ctxt,
                                   test_share,
                                   access_rules=access_rules,
                                   add_rules=add_rules,
                                   delete_rules=delete_rules)

        allow_params = (
            '%s/%s/%s/%s' % (
                'fake_pool',
                'Local',
                'fake_project',
                test_share['name'],
            ),
            test_share['share_proto'],
            'ip',
            'some-ip',
            const.ACCESS_LEVEL_RW,
        )
        if not (add_rules or delete_rules):
            clear_params = (
                '%s/%s/%s/%s' % (
                    'fake_pool',
                    'Local',
                    'fake_project',
                    test_share['name'],
                ),
                test_share['share_proto'],
            )
            mock_api.assert_has_calls([mock.call('clearAccessRules',
                                                 clear_params),
                                       mock.call(call_name,
                                                 allow_params)])
            mock_params.assert_called_with(test_share)
        else:
            mock_api.assert_called_once_with(call_name, allow_params)
            mock_params.assert_called_once_with(test_share)

    @ddt.data({'path': r'\\some-ip\shareName', 'share_proto': 'CIFS',
               'host': 'some-ip'},
              {'path': 'some-ip:shareName', 'share_proto': 'NFS',
               'host': 'some-ip'},
              {'path': 'some-ip:shareName', 'share_proto': 'NFS',
               'host': None})
    @ddt.unpack
    def test_get_location_path(self, path, share_proto, host):
        self._driver._hostname = 'some-ip'

        result = self._driver._get_location_path('shareName',
                                                 share_proto,
                                                 host)
        expected = {
            'is_admin_only': False,
            'metadata': {
                'preferred': True,
            },
            'path': path,
        }
        self.assertEqual(expected, result)

    def test_get_location_path_fail(self):
        self.assertRaises(exception.InvalidInput,
                          self._driver._get_location_path,
                          'shareName',
                          'SOME',
                          'some-ip')

    def test_get_network_allocations_number(self):
        result = self._driver.get_network_allocations_number()

        expected = 0
        self.assertEqual(expected, result)


class TegileAPIExecutorTestCase(test.TestCase):
    def setUp(self):
        self._api = tegile.TegileAPIExecutor("TestCase",
                                             test_config.tegile_nas_server,
                                             test_config.tegile_nas_login,
                                             test_config.tegile_nas_password)
        super(TegileAPIExecutorTestCase, self).setUp()

    def test_send_api_post(self):
        json_output = {'value': 'abc'}

        self.mock_object(requests, 'post',
                         mock.Mock(return_value=FakeResponse(200,
                                                             json_output)))
        result = self._api(method="Test", request_type='post', params='[]',
                           fine_logging=True)

        self.assertEqual(json_output, result)

    def test_send_api_get(self):
        json_output = {'value': 'abc'}

        self.mock_object(requests, 'get',
                         mock.Mock(return_value=FakeResponse(200,
                                                             json_output)))

        result = self._api(method="Test",
                           request_type='get',
                           fine_logging=False)

        self.assertEqual(json_output, result)

    def test_send_api_get_fail(self):
        self.mock_object(requests, 'get',
                         mock.Mock(return_value=FakeResponse(404, [])))

        self.assertRaises(exception.TegileAPIException,
                          self._api,
                          method="Test",
                          request_type='get',
                          fine_logging=False)

    def test_send_api_value_error_fail(self):
        json_output = {'value': 'abc'}

        self.mock_object(requests, 'post',
                         mock.Mock(return_value=FakeResponse(200,
                                                             json_output)))
        self.mock_object(FakeResponse, 'json',
                         mock.Mock(side_effect=ValueError))

        result = self._api(method="Test",
                           request_type='post',
                           fine_logging=False)

        expected = ''
        self.assertEqual(expected, result)



# Copyright 2011 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Tests For Scheduler Host Filters.
"""
import ddt

from manila.scheduler.filters import extra_specs_ops
from manila import test


@ddt.ddt
class ExtraSpecsOpsTestCase(test.TestCase):
    def _do_extra_specs_ops_test(self, value, req, matches):
        assertion = self.assertTrue if matches else self.assertFalse
        assertion(extra_specs_ops.match(value, req))

    @ddt.unpack
    @ddt.data(
        ('1', '1', True),
        ('', '1', False),
        ('3', '1', False),
        ('222', '2', False),
        ('4', '> 2', False),
        ('123', '= 123', True),
        ('124', '= 123', True),
        ('34', '=234', False),
        ('34', '=', False),
        ('123', 's== 123', True),
        ('1234', 's== 123', False),
        ('1234', 's!= 123', True),
        ('123', 's!= 123', False),
        ('1000', 's>= 234', False),
        ('1234', 's<= 1000', False),
        ('2', 's< 12', False),
        ('12', 's> 2', False),
        ('12311321', '<in> 11', True),
        ('12311321', '<in> 12311321', True),
        ('12311321', '<in> 12311321 <in>', True),
        ('12310321', '<in> 11', False),
        ('12310321', '<in> 11 <in>', False),
        ('abc', '<in> ABC', True),
        (True, 'True', True),
        (True, '<is> True', True),
        (True, '<is> False', False),
        (False, 'False', True),
        (False, '<is> False', True),
        (False, '<is> True', False),
        (False, 'Nonsense', False),
        (False, '<is> Nonsense', True),
        (True, 'False', False),
        (False, 'True', False),
        ('12', '<or> 11 <or> 12', True),
        ('13', '<or> 11 <or> 12', False),
        ('13', '<or> 11 <or> 12 <or>', False),
        ('abc', '<or> ABC <or> def', True),
        ('2', '<= 10', True),
        ('3', '<= 2', False),
        ('3', '>= 1', True),
        ('2', '>= 3', False),
        ('nfs', 'NFS', True),
        ('NFS', 'nfs', True),
        ('cifs', 'nfs', False),
    )
    def test_extra_specs_matches_simple(self, value, req, matches):
        self._do_extra_specs_ops_test(
            value, req, matches)



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Add DB support for share instance export locations metadata.

Revision ID: dda6de06349
Revises: 323840a08dc4
Create Date: 2015-11-30 13:50:15.914232

"""

# revision identifiers, used by Alembic.
revision = 'dda6de06349'
down_revision = '323840a08dc4'

from alembic import op
from oslo_log import log
from oslo_utils import uuidutils
import sqlalchemy as sa


SI_TABLE_NAME = 'share_instances'
EL_TABLE_NAME = 'share_instance_export_locations'
ELM_TABLE_NAME = 'share_instance_export_locations_metadata'
LOG = log.getLogger(__name__)


def upgrade():
    try:
        meta = sa.MetaData()
        meta.bind = op.get_bind()

        # Add new 'is_admin_only' column in export locations table that will be
        # used for hiding admin export locations from common users in API.
        op.add_column(
            EL_TABLE_NAME,
            sa.Column('is_admin_only', sa.Boolean, default=False))

        # Create new 'uuid' column as String(36) in export locations table
        # that will be used for API.
        op.add_column(
            EL_TABLE_NAME,
            sa.Column('uuid', sa.String(36), unique=True),
        )

        # Generate UUID for each existing export location.
        el_table = sa.Table(
            EL_TABLE_NAME, meta,
            sa.Column('id', sa.Integer),
            sa.Column('uuid', sa.String(36)),
            sa.Column('is_admin_only', sa.Boolean),
        )
        for record in el_table.select().execute():
            # pylint: disable=no-value-for-parameter
            el_table.update().values(
                is_admin_only=False,
                uuid=uuidutils.generate_uuid(),
            ).where(
                el_table.c.id == record.id,
            ).execute()

        # Make new 'uuid' column in export locations table not nullable.
        op.alter_column(
            EL_TABLE_NAME,
            'uuid',
            existing_type=sa.String(length=36),
            nullable=False,
        )
    except Exception:
        LOG.error("Failed to update '%s' table!",
                  EL_TABLE_NAME)
        raise

    try:
        op.create_table(
            ELM_TABLE_NAME,
            sa.Column('id', sa.Integer, primary_key=True),
            sa.Column('created_at', sa.DateTime),
            sa.Column('updated_at', sa.DateTime),
            sa.Column('deleted_at', sa.DateTime),
            sa.Column('deleted', sa.Integer),
            sa.Column('export_location_id', sa.Integer,
                      sa.ForeignKey('%s.id' % EL_TABLE_NAME,
                                    name="elm_id_fk"), nullable=False),
            sa.Column('key', sa.String(length=255), nullable=False),
            sa.Column('value', sa.String(length=1023), nullable=False),
            sa.UniqueConstraint('export_location_id', 'key', 'deleted',
                                name="elm_el_id_uc"),
            mysql_engine='InnoDB',
        )
    except Exception:
        LOG.error("Failed to create '%s' table!", ELM_TABLE_NAME)
        raise


def downgrade():
    try:
        op.drop_table(ELM_TABLE_NAME)
    except Exception:
        LOG.error("Failed to drop '%s' table!", ELM_TABLE_NAME)
        raise

    try:
        op.drop_column(EL_TABLE_NAME, 'is_admin_only')
        op.drop_column(EL_TABLE_NAME, 'uuid')
    except Exception:
        LOG.error("Failed to update '%s' table!", EL_TABLE_NAME)
        raise



# Copyright (c) 2016 Hitachi Data Systems
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manila.api import common


class ViewBuilder(common.ViewBuilder):
    _collection_name = "share_snapshot_export_locations"

    def _get_view(self, request, export_location, detail=False):
        context = request.environ['manila.context']

        result = {
            'share_snapshot_export_location': {
                'id': export_location['id'],
                'path': export_location['path'],
                'links': self._get_links(request, export_location['id']),
            }
        }

        ss_el = result['share_snapshot_export_location']
        if context.is_admin:
            ss_el['share_snapshot_instance_id'] = (
                export_location['share_snapshot_instance_id'])
            ss_el['is_admin_only'] = export_location['is_admin_only']

        if detail:
            ss_el['created_at'] = export_location['created_at']
            ss_el['updated_at'] = export_location['updated_at']

        return result

    def list_export_locations(self, request, export_locations):

        context = request.environ['manila.context']

        result = {self._collection_name: []}
        for export_location in export_locations:
            if context.is_admin or not export_location['is_admin_only']:
                result[self._collection_name].append(self._get_view(
                    request,
                    export_location)['share_snapshot_export_location'])
            else:
                continue

        return result

    def detail_export_location(self, request, export_location):
        return self._get_view(request, export_location, detail=True)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from manila.scheduler.evaluator import evaluator
from manila.scheduler.filters import base_host
from manila.scheduler import utils


LOG = logging.getLogger(__name__)


class DriverFilter(base_host.BaseHostFilter):
    """DriverFilter filters hosts based on a 'filter function' and metrics.

    DriverFilter filters based on share host's provided 'filter function'
    and metrics.
    """

    def host_passes(self, host_state, filter_properties):
        """Determines whether a host has a passing filter_function or not."""
        stats = self._generate_stats(host_state, filter_properties)

        LOG.debug("Driver Filter: Checking host '%s'",
                  stats['host_stats']['host'])
        result = self._check_filter_function(stats)
        LOG.debug("Result: %s", result)
        LOG.debug("Done checking host '%s'", stats['host_stats']['host'])

        return result

    def _check_filter_function(self, stats):
        """Checks if a share passes a host's filter function.

           Returns a tuple in the format (filter_passing, filter_invalid).
           Both values are booleans.
        """

        if stats['filter_function'] is None:
            LOG.debug("Filter function not set :: passing host.")
            return True

        try:
            filter_result = self._run_evaluator(stats['filter_function'],
                                                stats)
        except Exception as ex:
            # Warn the admin for now that there is an error in the
            # filter function.
            LOG.warning("Error in filtering function "
                        "'%(function)s' : '%(error)s' :: failing host.",
                        {'function': stats['filter_function'],
                         'error': ex, })
            return False

        msg = "Filter function result for host %(host)s: %(result)s."
        args = {'host': stats['host_stats']['host'],
                'result': str(filter_result)}
        LOG.info(msg, args)

        return filter_result

    def _run_evaluator(self, func, stats):
        """Evaluates a given function using the provided available stats."""
        host_stats = stats['host_stats']
        host_caps = stats['host_caps']
        extra_specs = stats['extra_specs']
        share_stats = stats['share_stats']

        result = evaluator.evaluate(
            func,
            extra=extra_specs,
            stats=host_stats,
            capabilities=host_caps,
            share=share_stats)

        return result

    def _generate_stats(self, host_state, filter_properties):
        """Generates statistics from host and share data."""

        filter_function = None

        if ('filter_function' in host_state.capabilities and
                host_state.capabilities['filter_function'] is not None):
            filter_function = str(
                host_state.capabilities['filter_function'])

        stats = utils.generate_stats(host_state, filter_properties)

        stats['filter_function'] = filter_function

        return stats



# Copyright 2016 Huawei Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from webob import exc

from manila.api.openstack import wsgi
from manila.api.views import share_snapshot_instances as instance_view
from manila import db
from manila import exception
from manila.i18n import _
from manila import share


class ShareSnapshotInstancesController(wsgi.Controller,
                                       wsgi.AdminActionsMixin):
    """The share snapshot instances API controller for the OpenStack API."""

    resource_name = 'share_snapshot_instance'
    _view_builder_class = instance_view.ViewBuilder

    def __init__(self):
        self.share_api = share.API()
        super(ShareSnapshotInstancesController, self).__init__()

    @wsgi.Controller.api_version('2.19')
    @wsgi.Controller.authorize
    def show(self, req, id):
        context = req.environ['manila.context']
        try:
            snapshot_instance = db.share_snapshot_instance_get(
                context, id)
        except exception.ShareSnapshotInstanceNotFound:
            msg = (_("Snapshot instance %s not found.") % id)
            raise exc.HTTPNotFound(explanation=msg)
        return self._view_builder.detail(req, snapshot_instance)

    @wsgi.Controller.api_version('2.19')
    @wsgi.Controller.authorize
    def index(self, req):
        """Return a summary list of snapshot instances."""
        return self._get_instances(req)

    @wsgi.Controller.api_version('2.19')
    @wsgi.Controller.authorize
    def detail(self, req):
        """Returns a detailed list of snapshot instances."""
        return self._get_instances(req, is_detail=True)

    def _get_instances(self, req, is_detail=False):
        """Returns list of snapshot instances."""
        context = req.environ['manila.context']
        snapshot_id = req.params.get('snapshot_id')

        instances = db.share_snapshot_instance_get_all_with_filters(
            context, {'snapshot_ids': snapshot_id})

        if is_detail:
            instances = self._view_builder.detail_list(req, instances)
        else:
            instances = self._view_builder.summary_list(req, instances)
        return instances

    @wsgi.Controller.api_version('2.19')
    @wsgi.action('reset_status')
    def reset_status(self, req, id, body):
        """Reset the 'status' attribute in the database."""
        return self._reset_status(req, id, body)

    def _update(self, *args, **kwargs):
        db.share_snapshot_instance_update(*args, **kwargs)


def create_resource():
    return wsgi.Resource(ShareSnapshotInstancesController())



# Copyright (c) 2015 Quobyte Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Quobyte driver helper.

Control Quobyte over its JSON RPC API.
"""

import requests
from requests import auth
from requests import codes

from oslo_log import log
from oslo_serialization import jsonutils
import six
import six.moves.urllib.parse as urlparse

from manila import exception
from manila import utils

LOG = log.getLogger(__name__)

ERROR_ENOENT = 2
ERROR_ENTITY_NOT_FOUND = -24
ERROR_GARBAGE_ARGS = -3


class JsonRpc(object):

    def __init__(self, url, user_credentials, ca_file=None, key_file=None,
                 cert_file=None):
        parsedurl = urlparse.urlparse(url)
        self._url = parsedurl.geturl()
        self._netloc = parsedurl.netloc
        self._ca_file = ca_file
        self._url_scheme = parsedurl.scheme
        if self._url_scheme == 'https':
            if not self._ca_file:
                self._ca_file = False
                LOG.warning(
                    "Will not verify the server certificate of the API service"
                    " because the CA certificate is not available.")
        self._id = 0
        self._credentials = auth.HTTPBasicAuth(
            user_credentials[0], user_credentials[1])
        self._key_file = key_file
        self._cert_file = cert_file

    @utils.synchronized('quobyte-request')
    def call(self, method_name, user_parameters, expected_errors=None):
        if expected_errors is None:
            expected_errors = []
        # prepare request
        self._id += 1
        parameters = {'retry': 'INFINITELY'}  # Backend specific setting
        if user_parameters:
            parameters.update(user_parameters)
        post_data = {
            'jsonrpc': '2.0',
            'method': method_name,
            'params': parameters,
            'id': six.text_type(self._id),
        }
        LOG.debug("Request payload to be send is: %s",
                  jsonutils.dumps(post_data))

        # send request
        if self._url_scheme == 'https':
            if self._cert_file:
                result = requests.post(url=self._url,
                                       json=post_data,
                                       auth=self._credentials,
                                       verify=self._ca_file,
                                       cert=(self._cert_file, self._key_file))
            else:
                result = requests.post(url=self._url,
                                       json=post_data,
                                       auth=self._credentials,
                                       verify=self._ca_file)
        else:
            result = requests.post(url=self._url,
                                   json=post_data,
                                   auth=self._credentials)

        # eval request response
        if result.status_code == codes['OK']:
            LOG.debug("Retrieved data from Quobyte backend: %s", result.text)
            response = result.json()
            return self._checked_for_application_error(response,
                                                       expected_errors)

        # If things did not work out provide error info
        LOG.debug("Backend request resulted in error: %s", result.text)
        result.raise_for_status()

    def _checked_for_application_error(self, result, expected_errors=None):
        if expected_errors is None:
            expected_errors = []
        if 'error' in result and result['error']:
            if 'message' in result['error'] and 'code' in result['error']:
                if result["error"]["code"] in expected_errors:
                    # hit an expected error, return empty result
                    return None
                else:
                    raise exception.QBRpcException(
                        result=result["error"]["message"],
                        qbcode=result["error"]["code"])
            else:
                raise exception.QBException(six.text_type(result["error"]))
        return result["result"]



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""The security service api."""

from oslo_log import log
from six.moves import http_client
import webob
from webob import exc

from manila.api import common
from manila.api.openstack import wsgi
from manila.api.views import security_service as security_service_views
from manila.common import constants
from manila import db
from manila import exception
from manila.i18n import _
from manila import policy
from manila import utils


RESOURCE_NAME = 'security_service'
LOG = log.getLogger(__name__)


class SecurityServiceController(wsgi.Controller):
    """The Shares API controller for the OpenStack API."""

    _view_builder_class = security_service_views.ViewBuilder

    def show(self, req, id):
        """Return data about the given security service."""
        context = req.environ['manila.context']
        try:
            security_service = db.security_service_get(context, id)
            policy.check_policy(context, RESOURCE_NAME, 'show',
                                security_service)
        except exception.NotFound:
            raise exc.HTTPNotFound()

        return self._view_builder.detail(req, security_service)

    def delete(self, req, id):
        """Delete a security service."""
        context = req.environ['manila.context']

        LOG.info("Delete security service with id: %s",
                 id, context=context)

        try:
            security_service = db.security_service_get(context, id)
        except exception.NotFound:
            raise exc.HTTPNotFound()

        share_nets = db.share_network_get_all_by_security_service(
            context, id)
        if share_nets:
            msg = _("Cannot delete security service. It is "
                    "assigned to share network(s)")
            raise exc.HTTPForbidden(explanation=msg)
        policy.check_policy(context, RESOURCE_NAME,
                            'delete', security_service)
        db.security_service_delete(context, id)

        return webob.Response(status_int=http_client.ACCEPTED)

    def index(self, req):
        """Returns a summary list of security services."""
        policy.check_policy(req.environ['manila.context'], RESOURCE_NAME,
                            'index')
        return self._get_security_services(req, is_detail=False)

    def detail(self, req):
        """Returns a detailed list of security services."""
        policy.check_policy(req.environ['manila.context'], RESOURCE_NAME,
                            'detail')
        return self._get_security_services(req, is_detail=True)

    def _get_security_services(self, req, is_detail):
        """Returns a transformed list of security services.

        The list gets transformed through view builder.
        """
        context = req.environ['manila.context']

        search_opts = {}
        search_opts.update(req.GET)

        # NOTE(vponomaryov): remove 'status' from search opts
        # since it was removed from security service model.
        search_opts.pop('status', None)
        if 'share_network_id' in search_opts:
            share_nw = db.share_network_get(context,
                                            search_opts['share_network_id'])
            security_services = share_nw['security_services']
            del search_opts['share_network_id']
        else:
            if context.is_admin and utils.is_all_tenants(search_opts):
                policy.check_policy(context, RESOURCE_NAME,
                                    'get_all_security_services')
                security_services = db.security_service_get_all(context)
            else:
                security_services = db.security_service_get_all_by_project(
                    context, context.project_id)
        search_opts.pop('all_tenants', None)
        common.remove_invalid_options(
            context,
            search_opts,
            self._get_security_services_search_options())
        if search_opts:
            results = []
            not_found = object()
            for ss in security_services:
                if all(ss.get(opt, not_found) == value for opt, value in
                       search_opts.items()):
                    results.append(ss)
            security_services = results

        limited_list = common.limited(security_services, req)

        if is_detail:
            security_services = self._view_builder.detail_list(
                req, limited_list)
            for ss in security_services['security_services']:
                share_networks = db.share_network_get_all_by_security_service(
                    context,
                    ss['id'])
                ss['share_networks'] = [sn['id'] for sn in share_networks]
        else:
            security_services = self._view_builder.summary_list(
                req, limited_list)
        return security_services

    def _get_security_services_search_options(self):
        return ('name', 'id', 'type', 'user',
                'server', 'dns_ip', 'domain', )

    def _share_servers_dependent_on_sn_exist(self, context,
                                             security_service_id):
        share_networks = db.share_network_get_all_by_security_service(
            context, security_service_id)

        for sn in share_networks:
            for sns in sn['share_network_subnets']:
                if 'share_servers' in sns and sns['share_servers']:
                    return True
        return False

    def update(self, req, id, body):
        """Update a security service."""
        context = req.environ['manila.context']

        if not body or 'security_service' not in body:
            raise exc.HTTPUnprocessableEntity()

        security_service_data = body['security_service']
        valid_update_keys = (
            'description',
            'name'
        )

        try:
            security_service = db.security_service_get(context, id)
            policy.check_policy(context, RESOURCE_NAME, 'update',
                                security_service)
        except exception.NotFound:
            raise exc.HTTPNotFound()

        if self._share_servers_dependent_on_sn_exist(context, id):
            for item in security_service_data:
                if item not in valid_update_keys:
                    msg = _("Cannot update security service %s. It is "
                            "attached to share network with share server "
                            "associated. Only 'name' and 'description' "
                            "fields are available for update.") % id
                    raise exc.HTTPForbidden(explanation=msg)

        policy.check_policy(context, RESOURCE_NAME, 'update', security_service)
        security_service = db.security_service_update(
            context, id, security_service_data)
        return self._view_builder.detail(req, security_service)

    def create(self, req, body):
        """Creates a new security service."""
        context = req.environ['manila.context']
        policy.check_policy(context, RESOURCE_NAME, 'create')

        if not self.is_valid_body(body, 'security_service'):
            raise exc.HTTPUnprocessableEntity()

        security_service_args = body['security_service']
        security_srv_type = security_service_args.get('type')
        allowed_types = constants.SECURITY_SERVICES_ALLOWED_TYPES
        if security_srv_type not in allowed_types:
            raise exception.InvalidInput(
                reason=(_("Invalid type %(type)s specified for security "
                          "service. Valid types are %(types)s") %
                        {'type': security_srv_type,
                         'types': ','.join(allowed_types)}))
        security_service_args['project_id'] = context.project_id
        security_service = db.security_service_create(
            context, security_service_args)

        return self._view_builder.detail(req, security_service)


def create_resource():
    return wsgi.Resource(SecurityServiceController())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""add_reservation_expire_index

Revision ID: 7d142971c4ef
Revises: d5db24264f5c
Create Date: 2017-03-02 09:19:27.114719

"""

# revision identifiers, used by Alembic.
revision = '7d142971c4ef'
down_revision = 'd5db24264f5c'

from alembic import op


INDEX_NAME = 'reservations_deleted_expire_idx'
TABLE_NAME = 'reservations'


def upgrade():
    op.create_index(INDEX_NAME, TABLE_NAME, ['deleted', 'expire'])


def downgrade():
    op.drop_index(INDEX_NAME, TABLE_NAME)



# Copyright 2010 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from collections import abc
import datetime

from manila.common import constants
from manila import exception as exc

FAKE_UUID = 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'
FAKE_UUIDS = {}


def stub_share(id, **kwargs):
    share = {
        'id': id,
        'share_proto': 'FAKEPROTO',
        'export_location': 'fake_location',
        'export_locations': ['fake_location', 'fake_location2'],
        'user_id': 'fakeuser',
        'project_id': 'fakeproject',
        'size': 1,
        'access_rules_status': 'active',
        'status': 'fakestatus',
        'display_name': 'displayname',
        'display_description': 'displaydesc',
        'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
        'snapshot_id': '2',
        'share_type_id': '1',
        'is_public': False,
        'snapshot_support': True,
        'create_share_from_snapshot_support': True,
        'revert_to_snapshot_support': False,
        'mount_snapshot_support': False,
        'replication_type': None,
        'has_replicas': False,
    }

    share_instance = {
        'host': 'fakehost',
        'availability_zone': 'fakeaz',
        'share_network_id': None,
        'share_server_id': 'fake_share_server_id',
        'access_rules_status': 'active',
        'share_type_id': '1',
    }
    if 'instance' in kwargs:
        share_instance.update(kwargs.pop('instance'))
    else:
        # remove any share instance kwargs so they don't go into the share
        for inst_key in share_instance.keys():
            if inst_key in kwargs:
                share_instance[inst_key] = kwargs.pop(inst_key)

    share.update(kwargs)

    # NOTE(ameade): We must wrap the dictionary in an class in order to stub
    # object attributes.
    class wrapper(abc.Mapping):
        def __getitem__(self, name):
            if hasattr(self, name):
                return getattr(self, name)
            return self.__dict__[name]

        def __iter__(self):
            return iter(self.__dict__)

        def __len__(self):
            return len(self.__dict__)

        def update(self, other):
            self.__dict__.update(other)

    fake_share = wrapper()
    fake_share.instance = {'id': "fake_instance_id"}
    fake_share.instance.update(share_instance)
    fake_share.update(share)

    # stub out is_busy based on task_state, this mimics what's in the Share
    # data model type
    if share.get('task_state') in constants.BUSY_TASK_STATES:
        fake_share.is_busy = True
    else:
        fake_share.is_busy = False

    fake_share.instances = [fake_share.instance]

    return fake_share


def stub_snapshot(id, **kwargs):
    snapshot = {
        'id': id,
        'share_id': 'fakeshareid',
        'share_proto': 'fakesnapproto',
        'export_location': 'fakesnaplocation',
        'user_id': 'fakesnapuser',
        'project_id': 'fakesnapproject',
        'host': 'fakesnaphost',
        'share_size': 1,
        'size': 1,
        'status': 'fakesnapstatus',
        'aggregate_status': 'fakesnapstatus',
        'share_name': 'fakesharename',
        'display_name': 'displaysnapname',
        'display_description': 'displaysnapdesc',
        'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
    }
    snapshot.update(kwargs)
    return snapshot


def stub_share_type(id, **kwargs):
    share_type = {
        'id': id,
        'name': 'fakesharetype',
        'description': 'fakesharetypedescription',
        'is_public': True,
    }
    share_type.update(kwargs)
    return share_type


def stub_share_type_get(context, share_type_id, **kwargs):
    return stub_share_type(share_type_id, **kwargs)


def stub_share_get(self, context, share_id, **kwargs):
    return stub_share(share_id, **kwargs)


def stub_share_get_notfound(self, context, share_id, **kwargs):
    raise exc.NotFound


def stub_share_delete(self, context, *args, **param):
    pass


def stub_share_update(self, context, *args, **param):
    share = stub_share('1')
    return share


def stub_snapshot_update(self, context, *args, **param):
    share = stub_share('1')
    return share


def stub_share_get_all_by_project(self, context, sort_key=None, sort_dir=None,
                                  search_opts={}):
    return [stub_share_get(self, context, '1')]


def stub_get_all_shares(self, context):
    return [stub_share(100, project_id='fake'),
            stub_share(101, project_id='superfake'),
            stub_share(102, project_id='superduperfake')]


def stub_snapshot_get(self, context, snapshot_id):
    return stub_snapshot(snapshot_id)


def stub_snapshot_get_notfound(self, context, snapshot_id):
    raise exc.NotFound


def stub_snapshot_create(self, context, share, display_name,
                         display_description):
    return stub_snapshot(200,
                         share_id=share['id'],
                         display_name=display_name,
                         display_description=display_description)


def stub_snapshot_delete(self, context, *args, **param):
    pass


def stub_snapshot_get_all_by_project(self, context, search_opts=None,
                                     limit=None, offset=None,
                                     sort_key=None, sort_dir=None):
    return [stub_snapshot_get(self, context, 2)]


def stub_share_group_snapshot_member(id, **kwargs):
    member = {
        'id': id,
        'share_id': 'fakeshareid',
        'share_instance_id': 'fakeshareinstanceid',
        'share_proto': 'fakesnapproto',
        'share_type_id': 'fake_share_type_id',
        'export_location': 'fakesnaplocation',
        'user_id': 'fakesnapuser',
        'project_id': 'fakesnapproject',
        'host': 'fakesnaphost',
        'share_size': 1,
        'size': 1,
        'status': 'fakesnapstatus',
        'created_at': datetime.datetime(1, 1, 1, 1, 1, 1),
    }
    member.update(kwargs)
    return member



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys
from unittest import mock

import ddt

from manila.cmd import share as manila_share
from manila import test

CONF = manila_share.CONF


@ddt.ddt
class ManilaCmdShareTestCase(test.TestCase):

    @ddt.data(None, [], ['foo', ], ['foo', 'bar', ])
    def test_main(self, backends):
        self.mock_object(manila_share.log, 'setup')
        self.mock_object(manila_share.log, 'register_options')
        self.mock_object(manila_share.utils, 'monkey_patch')
        self.mock_object(manila_share.service, 'process_launcher')
        self.mock_object(manila_share.service.Service, 'create')
        self.launcher = manila_share.service.process_launcher.return_value
        self.mock_object(self.launcher, 'launch_service')
        self.mock_object(self.launcher, 'wait')
        self.server = manila_share.service.Service.create.return_value
        fake_host = 'fake.host'
        CONF.set_override('enabled_share_backends', backends)
        CONF.set_override('host', fake_host)
        sys.argv = ['manila-share']

        manila_share.main()

        manila_share.log.setup.assert_called_once_with(CONF, "manila")
        manila_share.log.register_options.assert_called_once_with(CONF)
        manila_share.utils.monkey_patch.assert_called_once_with()
        manila_share.service.process_launcher.assert_called_once_with()
        self.launcher.wait.assert_called_once_with()

        if backends:
            manila_share.service.Service.create.assert_has_calls([
                mock.call(
                    host=fake_host + '@' + backend,
                    service_name=backend,
                    binary='manila-share',
                    coordination=True,
                ) for backend in backends
            ])
            self.launcher.launch_service.assert_has_calls([
                mock.call(self.server) for backend in backends])
        else:
            manila_share.service.Service.create.assert_called_once_with(
                binary='manila-share')
            self.launcher.launch_service.assert_called_once_with(self.server)



# Copyright (c) 2019 Infortrend Technology, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class InfortrendNASTestData(object):

    fake_share_id = ['5a0aa06e-1c57-4996-be46-b81e360e8866',  # NFS
                     'aac4fe64-7a9c-472a-b156-9adbb50b4d29']  # CIFS

    fake_share_name = [fake_share_id[0].replace('-', ''),
                       fake_share_id[1].replace('-', '')]

    fake_channel_ip = ['172.27.112.223', '172.27.113.209']

    fake_service_status_data = ('(64175, 1234, 272, 0)\n\n'
                                '{"cliCode": '
                                '[{"Return": "0x0000", "CLI": "Successful"}], '
                                '"returnCode": [], '
                                '"data": '
                                '[{"A": '
                                '{"NFS": '
                                '{"displayName": "NFS", '
                                '"state_time": "2017-05-04 14:19:53", '
                                '"enabled": true, '
                                '"cpu_rate": "0.0", '
                                '"mem_rate": "0.0", '
                                '"state": "exited", '
                                '"type": "share"}}}]}\n\n')

    fake_folder_status_data = ('(64175, 1234, 1017, 0)\n\n'
                               '{"cliCode": '
                               '[{"Return": "0x0000", "CLI": "Successful"}], '
                               '"returnCode": [], '
                               '"data": '
                               '[{"utility": "1.00", '
                               '"used": "33886208", '
                               '"subshare": true, '
                               '"share": false, '
                               '"worm": "", '
                               '"free": "321931374592", '
                               '"fsType": "xfs", '
                               '"owner": "A", '
                               '"readOnly": false, '
                               '"modifyTime": "2017-04-27 16:16", '
                               '"directory": "/share-pool-01/LV-1", '
                               '"volumeId": "6541BAFB2E6C57B6", '
                               '"mounted": true, '
                               '"size": "321965260800"}, '
                               '{"utility": "1.00", '
                               '"used": "33779712", '
                               '"subshare": false, '
                               '"share": false, '
                               '"worm": "", '
                               '"free": "107287973888", '
                               '"fsType": "xfs", '
                               '"owner": "A", '
                               '"readOnly": false, '
                               '"modifyTime": "2017-04-27 15:45", '
                               '"directory": "/share-pool-02/LV-1", '
                               '"volumeId": "147A8FB67DA39914", '
                               '"mounted": true, '
                               '"size": "107321753600"}]}\n\n')

    fake_nfs_status_off = [{
        'A': {
            'NFS': {
                'displayName': 'NFS',
                'state_time': '2017-05-04 14:19:53',
                'enabled': False,
                'cpu_rate': '0.0',
                'mem_rate': '0.0',
                'state': 'exited',
                'type': 'share',
            }
        }
    }]

    fake_folder_status = [{
        'utility': '1.00',
        'used': '33886208',
        'subshare': True,
        'share': False,
        'worm': '',
        'free': '321931374592',
        'fsType': 'xfs',
        'owner': 'A',
        'readOnly': False,
        'modifyTime': '2017-04-27 16:16',
        'directory': '/share-pool-01/LV-1',
        'volumeId': '6541BAFB2E6C57B6',
        'mounted': True,
        'size': '321965260800'}, {
        'utility': '1.00',
        'used': '33779712',
        'subshare': False,
        'share': False,
        'worm': '',
        'free': '107287973888',
        'fsType': 'xfs',
        'owner': 'A',
        'readOnly': False,
        'modifyTime': '2017-04-27 15:45',
        'directory': '/share-pool-02/LV-1',
        'volumeId': '147A8FB67DA39914',
        'mounted': True,
        'size': '107321753600',
    }]

    def fake_get_channel_status(self, ch1_status='UP'):
        return [{
            'datalink': 'mgmt0',
            'status': 'UP',
            'typeConfig': 'DHCP',
            'IP': '172.27.112.125',
            'MAC': '00:d0:23:00:15:a6',
            'netmask': '255.255.240.0',
            'type': 'dhcp',
            'gateway': '172.27.127.254'}, {
            'datalink': 'CH0',
            'status': 'UP',
            'typeConfig': 'DHCP',
            'IP': self.fake_channel_ip[0],
            'MAC': '00:d0:23:80:15:a6',
            'netmask': '255.255.240.0',
            'type': 'dhcp',
            'gateway': '172.27.127.254'}, {
            'datalink': 'CH1',
            'status': ch1_status,
            'typeConfig': 'DHCP',
            'IP': self.fake_channel_ip[1],
            'MAC': '00:d0:23:40:15:a6',
            'netmask': '255.255.240.0',
            'type': 'dhcp',
            'gateway': '172.27.127.254'}, {
            'datalink': 'CH2',
            'status': 'DOWN',
            'typeConfig': 'DHCP',
            'IP': '',
            'MAC': '00:d0:23:c0:15:a6',
            'netmask': '',
            'type': '',
            'gateway': ''}, {
            'datalink': 'CH3',
            'status': 'DOWN',
            'typeConfig': 'DHCP',
            'IP': '',
            'MAC': '00:d0:23:20:15:a6',
            'netmask': '',
            'type': '',
            'gateway': '',
        }]

    fake_fquota_status = [{
        'quota': '21474836480',
        'used': '0',
        'name': 'test-folder',
        'type': 'subfolder',
        'id': '537178178'}, {
        'quota': '32212254720',
        'used': '0',
        'name': fake_share_name[0],
        'type': 'subfolder',
        'id': '805306752'}, {
        'quota': '53687091200',
        'used': '21474836480',
        'name': fake_share_name[1],
        'type': 'subfolder',
        'id': '69'}, {
        'quota': '94091997184',
        'used': '0',
        'type': 'subfolder',
        'id': '70',
        "name": 'test-folder-02'
    }]

    fake_fquota_status_with_no_settings = []

    def fake_get_share_status_nfs(self, status=False):
        fake_share_status_nfs = [{
            'ftp': False,
            'cifs': False,
            'oss': False,
            'sftp': False,
            'nfs': status,
            'directory': '/LV-1/share-pool-01/' + self.fake_share_name[0],
            'exist': True,
            'afp': False,
            'webdav': False
        }]
        if status:
            fake_share_status_nfs[0]['nfs_detail'] = {
                'hostList': [{
                    'uid': '65534',
                    'insecure': 'insecure',
                    'squash': 'all',
                    'access': 'ro',
                    'host': '*',
                    'gid': '65534',
                    'mode': 'async',
                    'no_subtree_check': 'no_subtree_check',
                }]
            }
        return fake_share_status_nfs

    def fake_get_share_status_cifs(self, status=False):
        fake_share_status_cifs = [{
            'ftp': False,
            'cifs': status,
            'oss': False,
            'sftp': False,
            'nfs': False,
            'directory': '/share-pool-01/LV-1/' + self.fake_share_name[1],
            'exist': True,
            'afp': False,
            'webdav': False
        }]
        if status:
            fake_share_status_cifs[0]['cifs_detail'] = {
                'available': True,
                'encrypt': False,
                'description': '',
                'sharename': 'cifs-01',
                'failover': '',
                'AIO': True,
                'priv': 'None',
                'recycle_bin': False,
                'ABE': True,
            }
        return fake_share_status_cifs

    fake_subfolder_data = [{
        'size': '6',
        'index': '34',
        'description': '',
        'encryption': '',
        'isEnd': False,
        'share': False,
        'volumeId': '6541BAFB2E6C57B6',
        'quota': '',
        'modifyTime': '2017-04-06 11:35',
        'owner': 'A',
        'path': '/share-pool-01/LV-1/UserHome',
        'subshare': True,
        'type': 'subfolder',
        'empty': False,
        'name': 'UserHome'}, {
        'size': '6',
        'index': '39',
        'description': '',
        'encryption': '',
        'isEnd': False,
        'share': False,
        'volumeId': '6541BAFB2E6C57B6',
        'quota': '21474836480',
        'modifyTime': '2017-04-27 15:44',
        'owner': 'A',
        'path': '/share-pool-01/LV-1/test-folder',
        'subshare': False,
        'type': 'subfolder',
        'empty': True,
        'name': 'test-folder'}, {
        'size': '6',
        'index': '45',
        'description': '',
        'encryption': '',
        'isEnd': False,
        'share': True,
        'volumeId': '6541BAFB2E6C57B6',
        'quota': '32212254720',
        'modifyTime': '2017-04-27 16:15',
        'owner': 'A',
        'path': '/share-pool-01/LV-1/' + fake_share_name[0],
        'subshare': False,
        'type': 'subfolder',
        'empty': True,
        'name': fake_share_name[0]}, {
        'size': '6',
        'index': '512',
        'description': '',
        'encryption': '',
        'isEnd': True,
        'share': True,
        'volumeId': '6541BAFB2E6C57B6',
        'quota': '53687091200',
        'modifyTime': '2017-04-27 16:16',
        'owner': 'A',
        'path': '/share-pool-01/LV-1/' + fake_share_name[1],
        'subshare': False,
        'type': 'subfolder',
        'empty': True,
        'name': fake_share_name[1]}, {
        'size': '6',
        'index': '777',
        'description': '',
        'encryption': '',
        'isEnd': False,
        'share': False,
        'volumeId': '6541BAFB2E6C57B6',
        'quota': '94091997184',
        'modifyTime': '2017-04-28 15:44',
        'owner': 'A',
        'path': '/share-pool-01/LV-1/test-folder-02',
        'subshare': False,
        'type': 'subfolder',
        'empty': True,
        'name': 'test-folder-02'
    }]

    fake_cifs_user_list = [{
        'Superuser': 'No',
        'Group': 'users',
        'Description': '',
        'Quota': 'none',
        'PWD Expiry Date': '2291-01-19',
        'Home Directory': '/share-pool-01/LV-1/UserHome/user01',
        'UID': '100001',
        'Type': 'Local',
        'Name': 'user01'}, {
        'Superuser': 'No',
        'Group': 'users',
        'Description': '',
        'Quota': 'none',
        'PWD Expiry Date': '2017-08-07',
        'Home Directory': '/share-pool-01/LV-1/UserHome/user02',
        'UID': '100002',
        'Type': 'Local',
        'Name': 'user02'
    }]

    fake_share_status_nfs_with_rules = [{
        'ftp': False,
        'cifs': False,
        'oss': False,
        'sftp': False,
        'nfs': True,
        'directory': '/share-pool-01/LV-1/' + fake_share_name[0],
        'exist': True,
        'nfs_detail': {
            'hostList': [{
                'uid': '65534',
                'insecure': 'insecure',
                'squash': 'all',
                'access': 'ro',
                'host': '*',
                'gid': '65534',
                'mode': 'async',
                'no_subtree_check':
                'no_subtree_check'}, {
                'uid': '65534',
                'insecure': 'insecure',
                'squash': 'all',
                'access': 'rw',
                'host': '172.27.1.1',
                'gid': '65534',
                'mode': 'async',
                'no_subtree_check': 'no_subtree_check'}, {
                'uid': '65534',
                'insecure': 'insecure',
                'squash': 'all',
                'access': 'rw',
                'host': '172.27.1.2',
                'gid': '65534',
                'mode': 'async',
                'no_subtree_check': 'no_subtree_check'}]
        },
        'afp': False,
        'webdav': False,
    }]

    fake_share_status_cifs_with_rules = [
        {
            'permission': {
                'Read': True,
                'Write': True,
                'Execute': True},
            'type': 'user',
            'id': '100001',
            'name': 'user01'
        }, {
            'permission': {
                'Read': True,
                'Write': False,
                'Execute': True},
            'type': 'user',
            'id': '100002',
            'name': 'user02'
        }, {
            'permission': {
                'Read': True,
                'Write': False,
                'Execute': True},
            'type': 'group@',
            'id': '100',
            'name': 'users'
        }, {
            'permission': {
                'Read': True,
                'Write': False,
                'Execute': True},
            'type': 'other@',
            'id': '',
            'name': ''
        }
    ]



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import datetime
import iso8601
from oslo_config import cfg
import webob

from manila.api.v2 import messages
from manila import context
from manila import exception
from manila.message import api as message_api
from manila.message import message_field
from manila import policy
from manila import test
from manila.tests.api import fakes
from manila.tests.api.v2 import stubs

CONF = cfg.CONF


class MessageApiTest(test.TestCase):
    def setUp(self):
        super(MessageApiTest, self).setUp()
        self.controller = messages.MessagesController()

        self.maxDiff = None
        self.ctxt = context.RequestContext('admin', 'fake', True)
        self.mock_object(policy, 'check_policy',
                         mock.Mock(return_value=True))

    def _expected_message_from_controller(self, id, **kwargs):
        message = stubs.stub_message(id, **kwargs)
        links = [
            {'href': 'http://localhost/share/v2/fake/messages/%s' % id,
             'rel': 'self'},
            {'href': 'http://localhost/share/fake/messages/%s' % id,
             'rel': 'bookmark'},
        ]
        return {
            'message': {
                'id': message.get('id'),
                'project_id': message.get('project_id'),
                'user_message': "%s: %s" % (
                    message_field.translate_action(message.get('action_id')),
                    message_field.translate_detail(message.get('detail_id'))),
                'request_id': message.get('request_id'),
                'action_id': message.get('action_id'),
                'detail_id': message.get('detail_id'),
                'created_at': message.get('created_at'),
                'message_level': message.get('message_level'),
                'expires_at': message.get('expires_at'),
                'links': links,
                'resource_type': message.get('resource_type'),
                'resource_id': message.get('resource_id'),
            }
        }

    def test_show(self):
        self.mock_object(message_api.API, 'get', stubs.stub_message_get)

        req = fakes.HTTPRequest.blank(
            '/v2/fake/messages/%s' % fakes.FAKE_UUID,
            version=messages.MESSAGES_BASE_MICRO_VERSION)
        req.environ['manila.context'] = self.ctxt

        res_dict = self.controller.show(req, fakes.FAKE_UUID)

        ex = self._expected_message_from_controller(fakes.FAKE_UUID)
        self.assertEqual(ex, res_dict)

    def test_show_with_resource(self):
        resource_type = "FAKE_RESOURCE"
        resource_id = "b1872cb2-4c5f-4072-9828-8a51b02926a3"
        fake_message = stubs.stub_message(fakes.FAKE_UUID,
                                          resource_type=resource_type,
                                          resource_id=resource_id)
        mock_get = mock.Mock(return_value=fake_message)
        self.mock_object(message_api.API, 'get', mock_get)

        req = fakes.HTTPRequest.blank(
            '/v2/fake/messages/%s' % fakes.FAKE_UUID,
            version=messages.MESSAGES_BASE_MICRO_VERSION,
            base_url='http://localhost/share/v2')
        req.environ['manila.context'] = self.ctxt

        res_dict = self.controller.show(req, fakes.FAKE_UUID)

        self.assertEqual(resource_type,
                         res_dict['message']['resource_type'])
        self.assertEqual(resource_id,
                         res_dict['message']['resource_id'])

    def test_show_not_found(self):
        fake_not_found = exception.MessageNotFound(message_id=fakes.FAKE_UUID)
        self.mock_object(message_api.API, 'get',
                         mock.Mock(side_effect=fake_not_found))

        req = fakes.HTTPRequest.blank(
            '/v2/fake/messages/%s' % fakes.FAKE_UUID,
            version=messages.MESSAGES_BASE_MICRO_VERSION,
            base_url='http://localhost/share/v2')
        req.environ['manila.context'] = self.ctxt

        self.assertRaises(webob.exc.HTTPNotFound, self.controller.show,
                          req, fakes.FAKE_UUID)

    def test_show_pre_microversion(self):
        self.mock_object(message_api.API, 'get', stubs.stub_message_get)

        req = fakes.HTTPRequest.blank('/v2/fake/messages/%s' % fakes.FAKE_UUID,
                                      version='2.35',
                                      base_url='http://localhost/share/v2')
        req.environ['manila.context'] = self.ctxt

        self.assertRaises(exception.VersionNotFoundForAPIMethod,
                          self.controller.show, req, fakes.FAKE_UUID)

    def test_delete(self):
        self.mock_object(message_api.API, 'get', stubs.stub_message_get)
        self.mock_object(message_api.API, 'delete')

        req = fakes.HTTPRequest.blank(
            '/v2/fake/messages/%s' % fakes.FAKE_UUID,
            version=messages.MESSAGES_BASE_MICRO_VERSION)
        req.environ['manila.context'] = self.ctxt

        resp = self.controller.delete(req, fakes.FAKE_UUID)

        self.assertEqual(204, resp.status_int)
        self.assertTrue(message_api.API.delete.called)

    def test_delete_not_found(self):
        fake_not_found = exception.MessageNotFound(message_id=fakes.FAKE_UUID)
        self.mock_object(message_api.API, 'get',
                         mock.Mock(side_effect=fake_not_found))

        req = fakes.HTTPRequest.blank(
            '/v2/fake/messages/%s' % fakes.FAKE_UUID,
            version=messages.MESSAGES_BASE_MICRO_VERSION)

        self.assertRaises(webob.exc.HTTPNotFound, self.controller.delete,
                          req, fakes.FAKE_UUID)

    def test_index(self):
        msg1 = stubs.stub_message(fakes.get_fake_uuid())
        msg2 = stubs.stub_message(fakes.get_fake_uuid())
        self.mock_object(message_api.API, 'get_all', mock.Mock(
                         return_value=[msg1, msg2]))
        req = fakes.HTTPRequest.blank(
            '/v2/fake/messages',
            version=messages.MESSAGES_BASE_MICRO_VERSION,
            base_url='http://localhost/share/v2')
        req.environ['manila.context'] = self.ctxt

        res_dict = self.controller.index(req)

        ex1 = self._expected_message_from_controller(msg1['id'])['message']
        ex2 = self._expected_message_from_controller(msg2['id'])['message']
        expected = {'messages': [ex1, ex2]}
        self.assertDictEqual(expected, res_dict)

    def test_index_with_limit_and_offset(self):
        msg2 = stubs.stub_message(fakes.get_fake_uuid())
        self.mock_object(message_api.API, 'get_all', mock.Mock(
                         return_value=[msg2]))
        req = fakes.HTTPRequest.blank(
            '/v2/fake/messages?limit=1&offset=1',
            version=messages.MESSAGES_BASE_MICRO_VERSION)
        req.environ['manila.context'] = self.ctxt

        res_dict = self.controller.index(req)

        ex2 = self._expected_message_from_controller(msg2['id'])['message']
        self.assertEqual([ex2], res_dict['messages'])

    def test_index_with_created_since_and_created_before(self):
        msg = stubs.stub_message(
            fakes.get_fake_uuid(),
            created_at=datetime.datetime(1900, 2, 1, 1, 1, 1,
                                         tzinfo=iso8601.UTC))
        self.mock_object(message_api.API, 'get_all', mock.Mock(
                         return_value=[msg]))
        req = fakes.HTTPRequest.blank(
            '/fake/messages?created_since=1900-01-01T01:01:01&'
            'created_before=1900-03-01T01:01:01',
            version=messages.MESSAGES_QUERY_BY_TIMESTAMP,
            base_url='http://localhost/share/v2')
        req.environ['manila.context'] = self.ctxt

        res_dict = self.controller.index(req)

        ex2 = self._expected_message_from_controller(
            msg['id'],
            created_at=datetime.datetime(1900, 2, 1, 1, 1, 1,
                                         tzinfo=iso8601.UTC))['message']
        self.assertEqual([ex2], res_dict['messages'])

    def test_index_with_invalid_time_format(self):
        req = fakes.HTTPRequest.blank(
            '/messages?created_since=invalid_time_str',
            version=messages.MESSAGES_QUERY_BY_TIMESTAMP,
            base_url='http://localhost/share/v2')
        req.environ['manila.context'] = self.ctxt
        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller.index, req)



# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright 2014 NetApp, Inc.
# Copyright 2014 Mirantis, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Unit Tests for remote procedure calls using queue
"""

from unittest import mock

import ddt
from oslo_config import cfg
from oslo_service import wsgi

from manila import context
from manila import db
from manila import exception
from manila import manager
from manila import service
from manila import test
from manila import utils

test_service_opts = [
    cfg.StrOpt("fake_manager",
               default="manila.tests.test_service.FakeManager",
               help="Manager for testing"),
    cfg.StrOpt("test_service_listen",
               help="Host to bind test service to"),
    cfg.IntOpt("test_service_listen_port",
               default=0,
               help="Port number to bind test service to"),
]

CONF = cfg.CONF
CONF.register_opts(test_service_opts)


class FakeManager(manager.Manager):
    """Fake manager for tests."""

    RPC_API_VERSION = "1.0"

    def __init__(self, host=None, db_driver=None, service_name=None):
        super(FakeManager, self).__init__(host=host, db_driver=db_driver)

    def test_method(self):
        return 'manager'


class ExtendedService(service.Service):
    def test_method(self):
        return 'service'


class ServiceManagerTestCase(test.TestCase):
    """Test cases for Services."""

    def test_message_gets_to_manager(self):
        serv = service.Service('test', 'test', 'test', CONF.fake_manager)
        serv.start()
        self.assertEqual('manager', serv.test_method())

    def test_override_manager_method(self):
        serv = ExtendedService('test', 'test', 'test', CONF.fake_manager)
        serv.start()
        self.assertEqual('service', serv.test_method())


class ServiceFlagsTestCase(test.TestCase):
    def test_service_enabled_on_create_based_on_flag(self):
        self.flags(enable_new_services=True)
        host = 'foo'
        binary = 'manila-fake'
        app = service.Service.create(host=host, binary=binary)
        app.start()
        app.stop()
        ref = db.service_get(context.get_admin_context(), app.service_id)
        db.service_destroy(context.get_admin_context(), app.service_id)
        self.assertFalse(ref['disabled'])

    def test_service_disabled_on_create_based_on_flag(self):
        self.flags(enable_new_services=False)
        host = 'foo'
        binary = 'manila-fake'
        app = service.Service.create(host=host, binary=binary)
        app.start()
        app.stop()
        ref = db.service_get(context.get_admin_context(), app.service_id)
        db.service_destroy(context.get_admin_context(), app.service_id)
        self.assertTrue(ref['disabled'])


def fake_service_get_by_args(*args, **kwargs):
    raise exception.NotFound()


def fake_service_get(*args, **kwargs):
    raise Exception()


host = 'foo'
binary = 'bar'
topic = 'test'
service_create = {
    'host': host,
    'binary': binary,
    'topic': topic,
    'report_count': 0,
    'availability_zone': 'nova',
}
service_ref = {
    'host': host,
    'binary': binary,
    'topic': topic,
    'report_count': 0,
    'availability_zone': {'name': 'nova'},
    'id': 1,
}


@ddt.ddt
class ServiceTestCase(test.TestCase):
    """Test cases for Services."""

    def test_create(self):
        app = service.Service.create(host='foo',
                                     binary='manila-fake',
                                     topic='fake')
        self.assertTrue(app)

    @ddt.data(True, False)
    def test_periodic_tasks(self, raise_on_error):
        serv = service.Service(host, binary, topic, CONF.fake_manager)
        self.mock_object(
            context,
            'get_admin_context',
            mock.Mock(side_effect=context.get_admin_context))
        self.mock_object(serv.manager, 'periodic_tasks')

        serv.periodic_tasks(raise_on_error=raise_on_error)

        context.get_admin_context.assert_called_once_with()
        serv.manager.periodic_tasks.assert_called_once_with(
            utils.IsAMatcher(context.RequestContext),
            raise_on_error=raise_on_error)

    @mock.patch.object(service.db, 'service_get_by_args',
                       mock.Mock(side_effect=fake_service_get_by_args))
    @mock.patch.object(service.db, 'service_create',
                       mock.Mock(return_value=service_ref))
    @mock.patch.object(service.db, 'service_get',
                       mock.Mock(side_effect=fake_service_get))
    def test_report_state_newly_disconnected(self):
        serv = service.Service(host, binary, topic, CONF.fake_manager)
        serv.start()
        serv.report_state()
        self.assertTrue(serv.model_disconnected)
        service.db.service_get_by_args.assert_called_once_with(
            mock.ANY, host, binary)
        service.db.service_create.assert_called_once_with(
            mock.ANY, service_create)
        service.db.service_get.assert_called_once_with(mock.ANY, mock.ANY)

    @mock.patch.object(service.db, 'service_get_by_args',
                       mock.Mock(side_effect=fake_service_get_by_args))
    @mock.patch.object(service.db, 'service_create',
                       mock.Mock(return_value=service_ref))
    @mock.patch.object(service.db, 'service_get',
                       mock.Mock(return_value=service_ref))
    @mock.patch.object(service.db, 'service_update',
                       mock.Mock(return_value=service_ref.
                                 update({'report_count': 1})))
    def test_report_state_newly_connected(self):
        serv = service.Service(host, binary, topic, CONF.fake_manager)
        serv.start()
        serv.model_disconnected = True
        serv.report_state()
        self.assertFalse(serv.model_disconnected)
        service.db.service_get_by_args.assert_called_once_with(
            mock.ANY, host, binary)
        service.db.service_create.assert_called_once_with(
            mock.ANY, service_create)
        service.db.service_get.assert_called_once_with(
            mock.ANY, service_ref['id'])
        service.db.service_update.assert_called_once_with(
            mock.ANY, service_ref['id'], mock.ANY)

    def test_report_state_service_not_ready(self):
        with mock.patch.object(service, 'db') as mock_db:
            mock_db.service_get.return_value = service_ref
            serv = service.Service(host, binary, topic, CONF.fake_manager)
            serv.manager.is_service_ready = mock.Mock(return_value=False)
            serv.start()
            serv.report_state()

            serv.manager.is_service_ready.assert_called_once()
            mock_db.service_update.assert_not_called()


class TestWSGIService(test.TestCase):

    def setUp(self):
        super(TestWSGIService, self).setUp()
        self.mock_object(wsgi.Loader, 'load_app')
        self.test_service = service.WSGIService("test_service")

    def test_service_random_port(self):
        self.assertEqual(0, self.test_service.port)
        self.test_service.start()
        self.assertNotEqual(0, self.test_service.port)
        self.test_service.stop()
        wsgi.Loader.load_app.assert_called_once_with("test_service")

    def test_reset_pool_size_to_default(self):
        self.test_service.start()

        # Stopping the service, which in turn sets pool size to 0
        self.test_service.stop()
        self.assertEqual(0, self.test_service.server._pool.size)

        # Resetting pool size to default
        self.test_service.reset()
        self.test_service.start()
        self.assertGreater(self.test_service.server._pool.size, 0)
        wsgi.Loader.load_app.assert_called_once_with("test_service")

    @mock.patch('oslo_service.wsgi.Server')
    @mock.patch('oslo_service.wsgi.Loader')
    def test_ssl_enabled(self, mock_loader, mock_server):
        self.override_config('osapi_share_use_ssl', True)

        service.WSGIService("osapi_share")
        mock_server.assert_called_once_with(mock.ANY, mock.ANY, mock.ANY,
                                            port=mock.ANY, host=mock.ANY,
                                            use_ssl=True)

        self.assertTrue(mock_loader.called)



# Copyright (c) 2021 NetApp, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_log import log as logging

from manila import exception
from manila.i18n import _
from manila.share import driver
from manila import utils as manila_utils

# LDAP error codes
LDAP_INVALID_CREDENTIALS = 49

LOG = logging.getLogger(__name__)


class SecurityServiceHelper(driver.ExecuteMixin):
    def __init__(self, *args, **kwargs):
        self.configuration = kwargs.pop("configuration", None)
        super(SecurityServiceHelper, self).__init__(*args, **kwargs)
        self.init_execute_mixin()

    def setup_security_service(self, share_server_id, security_service):
        msg = ("Setting up the security service %(service)s for share server "
               "%(server_id)s")
        msg_args = {
            'service': security_service['id'],
            'server_id': share_server_id
        }
        LOG.debug(msg, msg_args)
        self.ldap_bind(share_server_id, security_service)

    def update_security_service(self, server_id, current_security_service,
                                new_security_service):
        msg = ("Updating the security service %(service)s for share server "
               "%(server_id)s")
        msg_args = {
            'service': new_security_service['id'],
            'server_id': server_id
        }
        LOG.debug(msg, msg_args)
        self.ldap_bind(server_id, new_security_service)

    def ldap_bind(self, share_server_id, security_service):
        ss_info = self.ldap_get_info(security_service)
        cmd = ["docker", "exec", "%s" % share_server_id, "ldapwhoami", "-x",
               "-H", "ldap://localhost:389", "-D",
               "cn=%s,dc=example,dc=com" % ss_info["ss_user"], "-w", "%s" %
               ss_info["ss_password"]]
        self.ldap_retry_operation(cmd, run_as_root=True)

    def ldap_get_info(self, security_service):
        if all(info in security_service for info in ("user", "password")):
            ss_user = security_service["user"]
            ss_password = security_service["password"]
        else:
            raise exception.ShareBackendException(
                _("LDAP requires user and password to be set for the bind "
                  "operation."))
        ss_info = {
            "ss_user": ss_user,
            "ss_password": ss_password,
        }
        return ss_info

    def ldap_retry_operation(self, cmd, run_as_root=True, timeout=30):
        interval = 5
        retries = int(timeout / interval) or 1

        @manila_utils.retry(retry_param=exception.ProcessExecutionError,
                            interval=interval,
                            retries=retries, backoff_rate=1)
        def try_ldap_operation():
            try:
                self._execute(*cmd, run_as_root=run_as_root)
            except exception.ProcessExecutionError as e:
                if e.exit_code == LDAP_INVALID_CREDENTIALS:
                    msg = _('LDAP credentials are invalid. '
                            'Aborting operation.')
                    LOG.warning(msg)
                    raise exception.ShareBackendException(msg=msg)
                else:
                    msg = _('Command has returned execution error.'
                            ' Will retry the operation.'
                            ' Error details: %s') % e.stderr
                    LOG.warning(msg)
                    raise exception.ProcessExecutionError()

        try:
            try_ldap_operation()
        except exception.ProcessExecutionError as e:
            msg = _("Unable to execute LDAP operation with success. "
                    "Retries exhausted. Error details: %s") % e.stderr
            LOG.exception(msg)
            raise exception.ShareBackendException(msg=msg)
